/**
 * \file id3v2-utils.cc
 *
 * Copyright (C) 2015-2021 Michael Herstine <sp1ff@pobox.com>
 *
 * This file is part of scribbu.
 *
 * scribbu is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * scribbu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with scribbu.  If not, see <http://www.gnu.org/licenses/>. *
 *
 *
 */

#include "unit.hh"

#include <scribbu/id3v2-utils.hh>

#include <boost/filesystem/fstream.hpp>
#include <boost/test/unit_test.hpp>

#include <scribbu/scribbu.hh>
#include <scribbu/pprinter.hh>

namespace fs = boost::filesystem;

BOOST_AUTO_TEST_CASE( test_maybe_read_id3 )
{
  using scribbu::id3v2_tag;

  const fs::path TEST_FILE_V2_2(get_data_directory() / "id3v2.2.tag");
  const fs::path TEST_FILE_V2_3(get_data_directory() / "lorca.mp3");
  const fs::path TEST_FILE_V2_4(get_data_directory() / "id3v2.4.tag");

  fs::ifstream ifs(TEST_FILE_V2_2);
  std::unique_ptr<id3v2_tag> ptag = scribbu::maybe_read_id3v2(ifs);

  BOOST_CHECK((bool)ptag);
  BOOST_CHECK(2 == ptag->version());
  BOOST_CHECK(0 == ptag->revision());
  BOOST_CHECK(2192 == ptag->size());
  BOOST_CHECK(0 == ptag->flags());
  boost::optional<bool> unsync = ptag->unsynchronised();
  BOOST_CHECK(unsync && ! *unsync);
  BOOST_CHECK("Mnemosyne's March (Demo)" == ptag->album()        );
  BOOST_CHECK("Murley Braid Quartet"     == ptag->artist()       );

  ifs.close();
  ifs.open(TEST_FILE_V2_3);
  ptag = scribbu::maybe_read_id3v2(ifs);

  BOOST_CHECK((bool)ptag);
  BOOST_CHECK(3 == ptag->version());
  BOOST_CHECK(0 == ptag->revision());
  BOOST_CHECK(452951 == ptag->size());
  BOOST_CHECK(0 == ptag->flags());
  unsync = ptag->unsynchronised();
  BOOST_CHECK(unsync && ! *unsync);

  BOOST_CHECK("Hell's Ditch [Expanded] (US Version)" == ptag->album());
  BOOST_CHECK("The Pogues" == ptag->artist());

  ifs.close();
  ifs.open(TEST_FILE_V2_4);
  ptag = scribbu::maybe_read_id3v2(ifs);

  BOOST_CHECK((bool)ptag);
  BOOST_CHECK(4 == ptag->version());
  BOOST_CHECK(0 == ptag->revision());
  BOOST_CHECK(1091 == ptag->size());
  BOOST_CHECK(0 == ptag->flags());
  unsync = ptag->unsynchronised();
  BOOST_CHECK(unsync && ! *unsync);
  BOOST_CHECK("Joao Gilberto" == ptag->artist());
  BOOST_CHECK("Acapulco" == ptag->title());
}

BOOST_AUTO_TEST_CASE( test_read_all_id3v2 )
{
  using namespace std;
  using namespace scribbu;

  const fs::path DATA1(get_data_directory() / "searchresults.dat");

  fs::ifstream ifs1(DATA1, fs::ifstream::binary);

  vector<unique_ptr<id3v2_tag>> tags;
  read_all_id3v2(ifs1, back_inserter(tags));
  BOOST_CHECK( tags.empty() );

}

/**
 * \brief Test multiple ID3v2 tags
 *
 * Test data 'lunch4bfast.mp3':
 *
 \code
 000000 49 44 33 03 00 00 00 00 0a 59 54 52 43 4b 00 00  >ID3......YTRCK..<
 000010 00 01 00 00 00 54 45 4e 43 00 00 00 01 40 00 00  >.....TENC....@..<
 000020 57 58 58 58 00 00 00 02 00 00 00 00 54 50 4f 53  >WXXX........TPOS<
 000030 00 00 00 01 00 00 00 54 43 4f 50 00 00 00 01 00  >.......TCOP.....<
 000040 00 00 54 4f 50 45 00 00 00 01 00 00 00 54 43 4f  >..TOPE.......TCO<
 000050 4d 00 00 00 01 00 00 00 43 4f 4d 4d 00 00 00 05  >M.......COMM....<
 000060 00 00 00 00 20 90 00 54 59 45 52 00 00 00 01 00  >.... ..TYER.....<
 000070 00 00 54 41 4c 42 00 00 00 01 00 00 00 54 50 45  >..TALB.......TPE<
 000080 31 00 00 00 01 00 00 00 54 49 54 32 00 00 00 01  >1.......TIT2....<
 000090 00 00 00 54 43 4f 4e 00 00 00 0b 00 00 00 45 6c  >...TCON.......El<
 0000a0 65 63 74 72 6f 6e 69 63 00 00 00 00 00 00 00 00  >ectronic........<
 0000b0 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00  >................<
 *
 000560 00 00 00 49 44 33 04 00 00 00 03 74 66 54 45 4e  >...ID3.....tfTEN<
 000570 43 00 00 00 12 00 00 00 69 54 75 6e 65 73 20 76  >C.......iTunes v<
 000580 35 2e 30 2e 30 2e 33 35 00 43 4f 4d 4d 00 00 00  >5.0.0.35.COMM...<
 000590 68 00 00 00 65 6e 67 69 54 75 6e 4e 4f 52 4d 00  >h...engiTunNORM.<
 0005a0 20 30 30 30 30 33 34 44 43 20 30 30 30 30 33 35  > 000034DC 000035<
 0005b0 34 39 20 30 30 30 31 32 30 32 39 20 30 30 30 31  >49 00012029 0001<
 0005c0 35 44 44 30 20 30 30 30 31 31 38 31 41 20 30 30  >5DD0 0001181A 00<
 0005d0 30 31 31 38 31 41 20 30 30 30 30 39 34 31 41 20  >01181A 0000941A <
 0005e0 30 30 30 30 39 39 30 38 20 30 30 30 32 41 45 44  >00009908 0002AED<
 0005f0 36 20 30 30 30 32 41 45 44 36 00 43 4f 4d 4d 00  >6 0002AED6.COMM.<
 000600 00 00 82 00 00 00 65 6e 67 69 54 75 6e 53 4d 50  >......engiTunSMP<
 000610 42 00 20 30 30 30 30 30 30 30 30 20 30 30 30 30  >B. 00000000 0000<
 000620 30 32 31 30 20 30 30 30 30 30 41 38 43 20 30 30  >0210 00000A8C 00<
 000630 30 30 30 30 30 30 30 30 43 39 34 41 36 34 20 30  >00000000C94A64 0<
 000640 30 30 30 30 30 30 30 20 30 30 30 30 30 30 30 30  >0000000 00000000<
 000650 20 30 30 30 30 30 30 30 30 20 30 30 30 30 30 30  > 00000000 000000<
 000660 30 30 20 30 30 30 30 30 30 30 30 20 30 30 30 30  >00 00000000 0000<
 000670 30 30 30 30 20 30 30 30 30 30 30 30 30 20 30 30  >0000 00000000 00<
 000680 30 30 30 30 30 30 00 43 4f 4d 4d 00 00 00 3f 00  >000000.COMM...?.<
 000690 00 00 65 6e 67 69 54 75 6e 65 73 5f 43 44 44 42  >..engiTunes_CDDB<
 0006a0 5f 31 00 32 37 30 35 31 41 30 35 2b 39 38 31 31  >_1.27051A05+9811<
 0006b0 33 2b 35 2b 31 35 30 2b 32 32 35 36 33 2b 34 30  >3+5+150+22563+40<
 0006c0 32 34 34 2b 36 30 32 31 35 2b 37 35 36 37 38 00  >244+60215+75678.<
 0006d0 43 4f 4d 4d 00 00 00 1e 00 00 00 65 6e 67 69 54  >COMM.......engiT<
 0006e0 75 6e 65 73 5f 43 44 44 42 5f 54 72 61 63 6b 4e  >unes_CDDB_TrackN<
 0006f0 75 6d 62 65 72 00 35 00 41 50 49 43 00 00 f8 53  >umber.5.APIC...S<
 000700 00 00 00 69 6d 61 67 65 2f 6a 70 65 67 00 00 00  >...image/jpeg...<
 000710 ff d8 ff e0 00 10 4a 46 49 46 00 01 02 01 00 48  >......JFIF.....H<
 000720 00 48 00 00 ff e1 13 12 45 78 69 66 00 00 4d 4d  >.H......Exif..MM<
 000730 00 2a 00 00 00 08 00 07 01 12 00 03 00 00 00 01  >.*..............<
 000740 00 01 00 00 01 1a 00 05 00 00 00 01 00 00 00 62  >...............b<
 000750 01 1b 00 05 00 00 00 01 00 00 00 6a 01 28 00 03  >...........j.(..<
 000760 00 00 00 01 00 02 00 00 01 31 00 02 00 00 00 1d  >.........1......<
 000770 00 00 00 72 01 32 00 02 00 00 00 14 00 00 00 8f  >...r.2..........<
 000780 87 69 00 04 00 00 00 01 00 00 00 a4 00 00 00 d0  >.i..............<
 000790 00 00 00 48 00 00 00 01 00 00 00 48 00 00 00 01  >...H.......H....<
 0007a0 41 64 6f 62 65 20 50 68 6f 74 6f 73 68 6f 70 20  >Adobe Photoshop <
 0007b0 43 53 20 4d 61 63 69 6e 74 6f 73 68 00 32 30 30  >CS Macintosh.200<
 0007c0 36 3a 30 31 3a 30 34 20 32 30 3a 32 31 3a 34 38  >6:01:04 20:21:48<
 0007d0 00 00 00 03 a0 01 00 03 00 00 00 01 00 01 00 00  >................<
 0007e0 a0 02 00 04 00 00 00 01 00 00 01 2c a0 03 00 04  >...........,....<
 0007f0 00 00 00 01 00 00 01 04 00 00 00 00 00 00 00 06  >................<
 000800 01 03 00 03 00 00 00 01 00 06 00 00 01 1a 00 05  >................<
 000810 00 00 00 01 00 00 01 1e 01 1b 00 05 00 00 00 01  >................<
 000820 00 00 01 26 01 28 00 03 00 00 00 01 00 02 00 00  >...&.(..........<
 000830 02 01 00 04 00 00 00 01 00 00 01 2e 02 02 00 04  >................<
 000840 00 00 00 01 00 00 11 dc 00 00 00 00 00 00 00 48  >...............H<
 000850 00 00 00 01 00 00 00 48 00 00 00 01 ff d8 ff e0  >.......H........<
 000860 00 10 4a 46 49 46 00 01 02 01 00 48 00 48 00 00  >..JFIF.....H.H..<
 000870 ff ed 00 0c 41 64 6f 62 65 5f 43 4d 00 01 ff ee  >....Adobe_CM....<
 000880 00 0e 41 64 6f 62 65 00 64 80 00 00 00 01 ff db  >..Adobe.d.......<
 000890 00 84 00 0c 08 08 08 09 08 0c 09 09 0c 11 0b 0a  >................<
 0008a0 0b 11 15 0f 0c 0c 0f 15 18 13 13 15 13 13 18 11  >................<
 \endcode
 *
 * Broken out:
 *
 \code
 000000 49 44 33 03 00                                   ID3 v2.3 tag
 000005                00                                no flags
 000006                   00 00 0a 59                    1369 bytes [1]
 00000a                               54 52 43 4b 00 00  >ID3......YTRCK..<
 000010 00 01 00 00 00 54 45 4e 43 00 00 00 01 40 00 00  >.....TENC....@..<
 000020 57 58 58 58 00 00 00 02 00 00 00 00 54 50 4f 53  >WXXX........TPOS<
 000030 00 00 00 01 00 00 00 54 43 4f 50 00 00 00 01 00  >.......TCOP.....<
 000040 00 00 54 4f 50 45 00 00 00 01 00 00 00 54 43 4f  >..TOPE.......TCO<
 000050 4d 00 00 00 01 00 00 00 43 4f 4d 4d 00 00 00 05  >M.......COMM....<
 000060 00 00 00 00 20 90 00 54 59 45 52 00 00 00 01 00  >.... ..TYER.....<
 000070 00 00 54 41 4c 42 00 00 00 01 00 00 00 54 50 45  >..TALB.......TPE<
 000080 31 00 00 00 01 00 00 00 54 49 54 32 00 00 00 01  >1.......TIT2....<
 000090 00 00 00 54 43 4f 4e 00 00 00 0b 00 00 00 45 6c  >...TCON.......El<
 0000a0 65 63 74 72 6f 6e 69 63 00 00 00 00 00 00 00 00  >ectronic........<
 0000b0 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00  >................<
 *
 000560 00 00 00
 000563          49 44 33 04 00                          ID3 v2.4 tag
 000568                         00                       no flags
 000569                            00 03 74 66           64102 bytes
 00056d                                        54 45 4e  TENC
 000570 43
 000571    00 00 00 12                                   18 bytes
 000575                00 00                             no flags
 000577                      00                          ISO-859-1
 000578                         69 54 75 6e 65 73 20 76  iTunes v
 000580 35 2e 30 2e 30 2e 33 35 00                       5.0.0.35.
 000589                            43 4f 4d 4d           COMM
 00058d                                        00 00 00  0x68 = 104 bytes
 000590 68
 000591    00 00                                         no flags
 000593          00                                      ISO-8859-1
 000594             65 6e 67                             eng
 000597                      69 54 75 6e 4e 4f 52 4d 00  iTunNORM
 0005a0 20 30 30 30 30 33 34 44 43 20 30 30 30 30 33 35   000034DC 000035
 0005b0 34 39 20 30 30 30 31 32 30 32 39 20 30 30 30 31  49 00012029 0001
 0005c0 35 44 44 30 20 30 30 30 31 31 38 31 41 20 30 30  5DD0 0001181A 00
 0005d0 30 31 31 38 31 41 20 30 30 30 30 39 34 31 41 20  01181A 0000941A
 0005e0 30 30 30 30 39 39 30 38 20 30 30 30 32 41 45 44  00009908 0002AED
 0005f0 36 20 30 30 30 32 41 45 44 36 00
 0005fb                                  43 4f 4d 4d     COMM
 0005ff                                              00  2 bytes <== ERROR
 000600 00 00 82                                         (really 0x82=130)
 000603          00 00                                   no flags
 000605                00                                ISO-8859-1
 000606                   65 6e 67                       eng
 000609                            69 54 75 6e 53 4d 50  iTunSMP
 000610 42 00 20 30 30 30 30 30 30 30 30 20 30 30 30 30  B. 00000000 0000
 000620 30 32 31 30 20 30 30 30 30 30 41 38 43 20 30 30  0210 00000A8C 00
 000630 30 30 30 30 30 30 30 30 43 39 34 41 36 34 20 30  00000000C94A64 0
 000640 30 30 30 30 30 30 30 20 30 30 30 30 30 30 30 30  0000000 00000000
 000650 20 30 30 30 30 30 30 30 30 20 30 30 30 30 30 30   00000000 000000
 000660 30 30 20 30 30 30 30 30 30 30 30 20 30 30 30 30  00 00000000 0000
 000670 30 30 30 30 20 30 30 30 30 30 30 30 30 20 30 30  0000 00000000 00
 000680 30 30 30 30 30 30 00                             000000.
 000687                      43 4f 4d 4d                 COMM
 00068b                                  00 00 00 3f     0x3f = 63 bytes
 00068f                                              00
 000690 00                                               no flags
 000691    00                                            ISO-8859-1
 000692       65 6e 67                                   eng
 000695                69 54 75 6e 65 73 5f 43 44 44 42  Tunes_CDDB
 0006a0 5f 31 00 32 37 30 35 31 41 30 35 2b 39 38 31 31  _1.27051A05+9811
 0006b0 33 2b 35 2b 31 35 30 2b 32 32 35 36 33 2b 34 30  3+5+150+22563+40
 0006c0 32 34 34 2b 36 30 32 31 35 2b 37 35 36 37 38 00  244+60215+75678.
 0006d0 43 4f 4d 4d                                      COMM
 0006d4             00 00 00 1e                          0x1e = 30 bytes
 0006d8                         00 00                    no flags
 0006da                               00                 ISO-8859-1
 0006db                                  65 6e 67        eng
 0006de                                           69 54  iT
 0006e0 75 6e 65 73 5f 43 44 44 42 5f 54 72 61 63 6b 4e  unes_CDDB_TrackN
 0006f0 75 6d 62 65 72 00 35 00                          umber.5.
 0006f8                         41 50 49 43              APIC
 0006fc                                     00 00 f8 53  (size) <== error
                                                   should be 0x00 03 70 53
 000700 00 00                                             no flags
 000702       00                                         ISO-8859-1
 000703          69 6d 61 67 65 2f 6a 70 65 67 00        image/jpeg.
 00070e                                           00     picture type
 00070f                                              00  description
 000710 ff d8 ff e0 00 10 4a 46 49 46 00 01 02 01 00 48  >......JFIF.....H<
 *
 00ff50 14 ea aa ff d9
 00ff55                54 49 54 32                       TIT2
 00ff59                            00 00 00 15           0x15 bytes
 00ff5d                                        00 00     no flags
 00ff5f                                              00  ISO-8859-1
 00ff60 4c 75 6e 63 68 20 66 6f 72 20 42 72 65 61 6b 66  Lunch for Breakf
 00ff70 61 73 74 00                                      ast.
 00ff74             54 50 45 31                          TPE1
 00ff78                         00 00 00 18              0x18 bytes
 00ff7c                                     00 00        no flags
 00ff7e                                           00     ISO-8859-1
 00ff7e                                              43  C
 00ff80 61 73 73 65 74 74 65 73 20 57 6f 6e 27 74 20 4c  assettes Won't L
 00ff90 69 73 74 65 6e 00                                isten.
 00ff96                   54 41 4c 42                    TALB
 00ff9a                               00 00 00 14        0x14 bytes
 00ff9e                                           00 00  no flags
 00ffa0 00                                               ISO-8859-1
 00ffa1    4e 6f 62 6f 64 79 27 73 20 4d 6f 76 69 6e 67  Nobody's Moving
 00ffb0 20 45 50 00
 00ffb4             54 52 43 4b                          TRCK
 00ffb8                         00 00 00 05              5 bytes
 00ffbc                                     00 00        no flags
 00ffbe                                           00     ISO-8859-1
 00ffbf                                              33  3
 00ffc0 2f 35 00                                         /5.
 00ffc3          54 44 52 43                             TDRC
 00ffc7                      00 00 00 06                 6 bytes
 00ffcb                                  00 00           no flags
 00ffcd                                        00        ISO-8859-1
 00ffce                                           32 30  20
 00ffd0 30 35 00                                         05
 00ffd3          ff fb b2 00 00 00 00 00 00 37 80 00 00  .........7...
 *
 \endcode
 *
 \code
 1.

 0x0a59 = b0000 1010 0101 1001
       => b 000 1010  101 1001
        = b00010101011001
        = b0000 0101 0101 1001
        = 0x05 59
        = 1369

 \endcode
 *
 \code
 2.

  0x00 03 74 66 = b0000 0011 0111 0100 0110 0110
               -> b 000 0011  111 0100  110 0110
                = b000001111101001100110
                = b0 0000 1111 1010 0110 0110
                = 0xfa66
                = 64102

 \endcode
 *
 * The second comment tag *is* in fact 0x82 bytes in size, but whoever
 * wrote the tag forgot to make the value sync-safe:
 *
 \code

  0x00 00 00 82 = b0000 0000 0000 0000 0000 0000 1000 0010
  -> 0... 000 0001 000 0010
  -> 0... 0000 0001 0000 0010
  -> 0... 0102 = 0x00 00 01 02
 \endcode
 *
 *
 * Test data 'duplicate_id3v2.mp3':
 *
 \code
 000000 49 44 33 03 00 00 00 00 1e 5d 54 41 4c 42 00 00  >ID3......]TALB..<
 000010 00 15 00 00 01 ff fe 41 00 6c 00 62 00 75 00 6d  >.......A.l.b.u.m<
 000020 00 58 00 58 00 58 00 58 00 54 50 45 31 00 00 00  >.X.X.X.X.TPE1...<
 000030 17 00 00 01 ff fe 41 00 72 00 74 00 69 00 73 00  >......A.r.t.i.s.<
 000040 74 00 58 00 58 00 58 00 58 00 54 49 54 32 00 00  >t.X.X.X.X.TIT2..<
 000050 00 15 00 00 01 ff fe 54 00 69 00 74 00 6c 00 65  >.......T.i.t.l.e<
 000060 00 58 00 58 00 58 00 58 00 50 52 49 56 00 00 00  >.X.X.X.X.PRIV...<
 000070 27 00 00 57 4d 2f 57 4d 43 6f 6c 6c 65 63 74 69  >'..WM/WMCollecti<
 000080 6f 6e 47 72 6f 75 70 49 44 00 6d 6b dd f3 57 a0  >onGroupID.mk..W.<
 000090 eb 40 ac 64 17 26 16 e6 60 a1 50 52 49 56 00 00  >.@.d.&..`.PRIV..<
 0000a0 00 8a 00 00 57 4d 2f 55 6e 69 71 75 65 46 69 6c  >....WM/UniqueFil<
 0000b0 65 49 64 65 6e 74 69 66 69 65 72 00 41 00 4d 00  >eIdentifier.A.M.<
 0000c0 47 00 61 00 5f 00 69 00 64 00 3d 00 52 00 20 00  >G.a._.i.d.=.R. .<
 0000d0 20 00 31 00 30 00 39 00 36 00 34 00 33 00 39 00  > .1.0.9.6.4.3.9.<
 0000e0 3b 00 41 00 4d 00 47 00 70 00 5f 00 69 00 64 00  >;.A.M.G.p._.i.d.<
 0000f0 3d 00 50 00 20 00 20 00 20 00 20 00 31 00 33 00  >=.P. . . . .1.3.<
 000100 34 00 38 00 34 00 3b 00 41 00 4d 00 47 00 74 00  >4.8.4.;.A.M.G.t.<
 000110 5f 00 69 00 64 00 3d 00 54 00 20 00 31 00 31 00  >_.i.d.=.T. .1.1.<
 000120 31 00 32 00 31 00 30 00 30 00 37 00 00 00 50 52  >1.2.1.0.0.7...PR<
 000130 49 56 00 00 00 14 00 00 57 4d 2f 50 72 6f 76 69  >IV......WM/Provi<
 000140 64 65 72 00 41 00 4d 00 47 00 00 00 50 52 49 56  >der.A.M.G...PRIV<
 000150 00 00 00 27 00 00 57 4d 2f 4d 65 64 69 61 43 6c  >...'..WM/MediaCl<
 000160 61 73 73 50 72 69 6d 61 72 79 49 44 00 bc 7d 60  >assPrimaryID..}`<
 000170 d1 23 e3 e2 4b 86 a1 48 a4 2a 28 44 1e 50 52 49  >.#..K..H.*(D.PRI<
 000180 56 00 00 00 22 00 00 57 4d 2f 57 4d 43 6f 6c 6c  >V..."..WM/WMColl<
 000190 65 63 74 69 6f 6e 49 44 00 6d 6b dd f3 57 a0 eb  >ectionID.mk..W..<
 0001a0 40 ac 64 17 26 16 e6 60 a1 50 52 49 56 00 00 00  >@.d.&..`.PRIV...<
 0001b0 1f 00 00 57 4d 2f 57 4d 43 6f 6e 74 65 6e 74 49  >...WM/WMContentI<
 0001c0 44 00 9f 36 29 24 05 6d 79 49 a2 e1 ec e4 9e f6  >D..6)$.myI......<
 0001d0 fc dd 50 52 49 56 00 00 00 29 00 00 57 4d 2f 4d  >..PRIV...)..WM/M<
 0001e0 65 64 69 61 43 6c 61 73 73 53 65 63 6f 6e 64 61  >ediaClassSeconda<
 0001f0 72 79 49 44 00 00 00 00 00 00 00 00 00 00 00 00  >ryID............<
 000200 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00  >................<
 *
 000f60 00 00 00 00 00 00 00 49 44 33 04 00 00 00 00 20  >.......ID3..... <
 000f70 00 54 49 54 32 00 00 00 31 00 00 01 ff fe 4a 00  >.TIT2...1.....J.<
 000f80 6f 00 20 00 4f 00 6e 00 65 00 73 00 20 00 55 00  >o. .O.n.e.s. .U.<
 000f90 67 00 6c 00 79 00 20 00 41 00 66 00 74 00 65 00  >g.l.y. .A.f.t.e.<
 000fa0 72 00 20 00 32 00 20 00 41 00 4d 00 54 50 45 31  >r. .2. .A.M.TPE1<
 000fb0 00 00 00 27 00 00 01 ff fe 4c 00 65 00 66 00 74  >...'.....L.e.f.t<
 000fc0 00 20 00 57 00 69 00 6e 00 67 00 20 00 46 00 61  >. .W.i.n.g. .F.a<
 000fd0 00 73 00 63 00 69 00 73 00 74 00 73 00 54 52 43  >.s.c.i.s.t.s.TRC<
 000fe0 4b 00 00 00 05 00 00 00 37 2f 31 31 54 41 4c 42  >K.......7/11TALB<
 000ff0 00 00 00 1b 00 00 01 ff fe 41 00 6c 00 6c 00 20  >.........A.l.l. <
 001000 00 46 00 69 00 72 00 65 00 64 00 20 00 55 00 70  >.F.i.r.e.d. .U.p<
 001010 00 54 50 4f 53 00 00 00 04 00 00 00 31 2f 31 54  >.TPOS.......1/1T<
 001020 44 52 43 00 00 00 17 00 00 01 ff fe 31 00 39 00  >DRC.........1.9.<
 001030 39 00 31 00 2d 00 30 00 31 00 2d 00 30 00 31 00  >9.1.-.0.1.-.0.1.<
 001040 54 43 4f 4e 00 00 00 0b 00 00 01 ff fe 50 00 75  >TCON.........P.u<
 001050 00 6e 00 6b 00 54 4d 45 44 00 00 00 1d 00 00 01  >.n.k.TMED.......<
 001060 ff fe 44 00 69 00 67 00 69 00 74 00 61 00 6c 00  >..D.i.g.i.t.a.l.<
 001070 20 00 4d 00 65 00 64 00 69 00 61 00 54 58 58 58  > .M.e.d.i.a.TXXX<
 001080 00 00 00 1b 00 00 01 ff fe 53 00 43 00 52 00 49  >.........S.C.R.I<
 001090 00 50 00 54 00 00 00 ff fe 4c 00 61 00 74 00 6e  >.P.T.....L.a.t.n<
 0010a0 00 54 58 58 58 00 00 00 65 00 00 01 ff fe 41 00  >.TXXX...e.....A.<
 0010b0 63 00 6f 00 75 00 73 00 74 00 69 00 64 00 20 00  >c.o.u.s.t.i.d. .<
 0010c0 49 00 64 00 00 00 ff fe 34 00 35 00 62 00 65 00  >I.d.....4.5.b.e.<
 0010d0 33 00 33 00 36 00 65 00 2d 00 35 00 31 00 64 00  >3.3.6.e.-.5.1.d.<
 0010e0 61 00 2d 00 34 00 66 00 62 00 38 00 2d 00 39 00  >a.-.4.f.b.8.-.9.<
 0010f0 64 00 35 00 64 00 2d 00 31 00 31 00 63 00 35 00  >d.5.d.-.1.1.c.5.<
 001100 32 00 64 00 37 00 64 00 38 00 30 00 61 00 31 00  >2.d.7.d.8.0.a.1.<
 001110 54 58 58 58 00 00 00 3d 00 00 01 ff fe 4d 00 75  >TXXX...=.....M.u<
 001120 00 73 00 69 00 63 00 42 00 72 00 61 00 69 00 6e  >.s.i.c.B.r.a.i.n<
 001130 00 7a 00 20 00 41 00 6c 00 62 00 75 00 6d 00 20  >.z. .A.l.b.u.m. <
 001140 00 54 00 79 00 70 00 65 00 00 00 ff fe 61 00 6c  >.T.y.p.e.....a.l<
 001150 00 62 00 75 00 6d 00 54 58 58 58 00 00 01 05 00  >.b.u.m.TXXX.....<
 001160 00 01 ff fe 4d 00 75 00 73 00 69 00 63 00 42 00  >....M.u.s.i.c.B.<
 001170 72 00 61 00 69 00 6e 00 7a 00 20 00 41 00 6c 00  >r.a.i.n.z. .A.l.<
 001180 62 00 75 00 6d 00 20 00 41 00 72 00 74 00 69 00  >b.u.m. .A.r.t.i.<
 001190 73 00 74 00 20 00 49 00 64 00 00 00 ff fe 31 00  >s.t. .I.d.....1.<
 0011a0 66 00 63 00 35 00 35 00 62 00 37 00 62 00 2d 00  >f.c.5.5.b.7.b.-.<
 0011b0 64 00 65 00 63 00 31 00 2d 00 34 00 38 00 30 00  >d.e.c.1.-.4.8.0.<
 0011c0 66 00 2d 00 38 00 35 00 39 00 39 00 2d 00 35 00  >f.-.8.5.9.9.-.5.<
 0011d0 63 00 31 00 61 00 63 00 38 00 33 00 31 00 33 00  >c.1.a.c.8.3.1.3.<
 0011e0 63 00 30 00 36 00 54 58 58 58 00 00 00 79 00 00  >c.0.6.TXXX...y..<
 0011f0 01 ff fe 4d 00 75 00 73 00 69 00 63 00 42 00 72  >...M.u.s.i.c.B.r<
 001200 00 61 00 69 00 6e 00 7a 00 20 00 41 00 72 00 74  >.a.i.n.z. .A.r.t<
 001210 00 69 00 73 00 74 00 20 00 49 00 64 00 00 00 ff  >.i.s.t. .I.d....<
 001220 fe 31 00 66 00 63 00 35 00 35 00 62 00 37 00 62  >.1.f.c.5.5.b.7.b<
 001230 00 2d 00 64 00 65 00 63 00 31 00 2d 00 34 00 38  >.-.d.e.c.1.-.4.8<
 001240 00 30 00 66 00 2d 00 38 00 35 00 39 00 39 00 2d  >.0.f.-.8.5.9.9.-<
 001250 00 35 00 63 00 31 00 61 00 63 00 38 00 33 00 31  >.5.c.1.a.c.8.3.1<
 001260 00 33 00 63 00 30 00 36 00 54 44 4f 52 00 00 00  >.3.c.0.6.TDOR...<
 001270 17 00 00 01 ff fe 31 00 39 00 39 00 31 00 2d 00  >......1.9.9.1.-.<
 001280 30 00 31 00 2d 00 30 00 31 00 54 53 4f 32 00 00  >0.1.-.0.1.TSO2..<
 001290 00 27 00 00 01 ff fe 4c 00 65 00 66 00 74 00 20  >.'.....L.e.f.t. <
 0012a0 00 57 00 69 00 6e 00 67 00 20 00 46 00 61 00 73  >.W.i.n.g. .F.a.s<
 0012b0 00 63 00 69 00 73 00 74 00 73 00 54 50 45 32 00  >.c.i.s.t.s.TPE2.<
 0012c0 00 00 27 00 00 01 ff fe 4c 00 65 00 66 00 74 00  >..'.....L.e.f.t.<
 0012d0 20 00 57 00 69 00 6e 00 67 00 20 00 46 00 61 00  > .W.i.n.g. .F.a.<
 0012e0 73 00 63 00 69 00 73 00 74 00 73 00 54 58 58 58  >s.c.i.s.t.s.TXXX<
 0012f0 00 00 01 07 00 00 01 ff fe 4d 00 75 00 73 00 69  >.........M.u.s.i<
 001300 00 63 00 42 00 72 00 61 00 69 00 6e 00 7a 00 20  >.c.B.r.a.i.n.z. <
 001310 00 52 00 65 00 6c 00 65 00 61 00 73 00 65 00 20  >.R.e.l.e.a.s.e. <
 001320 00 47 00 72 00 6f 00 75 00 70 00 20 00 49 00 64  >.G.r.o.u.p. .I.d<
 001330 00 00 00 ff fe 31 00 62 00 34 00 66 00 62 00 38  >.....1.b.4.f.b.8<
 001340 00 64 00 66 00 2d 00 61 00 33 00 63 00 61 00 2d  >.d.f.-.a.3.c.a.-<
 001350 00 34 00 64 00 30 00 30 00 2d 00 62 00 32 00 32  >.4.d.0.0.-.b.2.2<
 001360 00 62 00 2d 00 38 00 61 00 35 00 61 00 32 00 32  >.b.-.8.a.5.a.2.2<
 001370 00 38 00 32 00 62 00 31 00 35 00 30 00 41 50 49  >.8.2.b.1.5.0.API<
 001380 43 00 00 10 5d 00 00 00 69 6d 61 67 65 2f 6a 70  >C...]...image/jp<
 001390 65 67 00 03 00 ff d8 ff e0 00 10 4a 46 49 46 00  >eg.........JFIF.<
 0013a0 01 01 01 00 c8 00 c8 00 00 ff db 00 43 00 08 06  >............C...<
 0013b0 06 07 06 05 08 07 07 07 09 09 08 0a 0c 14 0d 0c  >................<
 0013c0 0b 0b 0c 19 12 13 0f 14 1d 1a 1f 1e 1d 1a 1c 1c  >................<
 0013d0 20 24 2e 27 20 22 2c 23 1c 1c 28 37 29 2c 30 31  > $.' ",#..(7),01<
 0013e0 34 34 34 1f 27 39 3d 38 32 3c 2e 33 34 32 ff db  >444.'9=82<.342..<
 0013f0 00 43 01 09 09 09 0c 0b 0c 18 0d 0d 18 32 21 1c  >.C...........2!.<
 001400 21 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32  >!222222222222222<
 001410 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32  >2222222222222222<
 *
 001430 32 32 32 ff c0 00 11 08 00 46 00 46 03 01 22 00  >222......F.F..".<
 001440 02 11 01 03 11 01 ff c4 00 1c 00 00 02 02 03 01  >................<
 001450 01 00 00 00 00 00 00 00 00 00 00 00 06 05 07 01  >................<
 001460 03 04 02 08 ff c4 00 42 10 00 01 03 02 05 00 05  >.......B........<
 001470 07 0a 01 0d 00 00 00 00 00 01 02 03 04 00 11 05  >................<
 001480 06 12 21 31 13 22 41 51 b1 07 14 23 34 61 73 d1  >..!1."AQ...#4as.<
 001490 15 24 32 33 53 71 72 81 91 92 16 25 42 43 52 54  >.$23Sqr....%BCRT<
 0014a0 62 a1 a2 b2 c1 d2 e1 f1 ff c4 00 1b 01 00 02 03  >b...............<
 0014b0 01 01 01 00 00 00 00 00 00 00 00 00 00 00 04 01  >................<
 0014c0 03 06 07 02 05 ff c4 00 29 11 00 02 01 01 04 0a  >........).......<
 0014d0 03 01 00 00 00 00 00 00 00 00 00 01 02 03 04 11  >................<
 0014e0 31 71 06 12 13 14 15 21 23 33 51 91 05 41 53 c1  >1q.....!#3Q..AS.<
 0014f0 ff da 00 0c 03 01 00 02 11 03 11 00 3f 00 41 79  >............?.Ay<
 001500 e4 b0 ca dd 50 25 29 17 36 a8 df e2 08 9f 66 f7  >....P%).6.....f.<
 001510 e8 3e 35 d7 88 6f 01 f1 fd c3 e1 4a 20 52 f4 e0  >.>5..o.....J R..<
 001520 a4 b9 9a ef 96 f9 0a f6 5a 91 8d 3b ae 68 64 4e  >........Z..;.hdN<
 001530 3b 19 76 b3 4f 5c f1 d5 1f 1a da ac 55 84 b7 ac  >;.v.O\......U...<
 001540 b6 ed b8 fa 23 63 dd cd 71 65 ec 1a 66 24 fa d5  >....#c..qe..f$..<
 001550 11 29 50 68 5d 6a 52 4a 85 c8 3a 45 85 c9 37 07  >.)Ph]jRJ..:E..7.<
 001560 60 3b 29 8a 36 57 c6 e2 b0 e2 bc c2 78 29 58 48  >`;).6W......x)XH<
 001570 52 90 b4 a5 63 fa a1 00 15 28 71 b9 b0 da ac d9  >R...c....(q.....<
 001580 44 f9 5c 72 d7 e5 7a 21 55 8d 47 46 ea 69 e0 38  >D.\r..z!U.GF.i.8<
 001590 e0 7c 6b cf cb d1 7e cd ef da 3e 35 30 ac 1b 0e  >.|k...~...>50...<
 0015a0 92 a5 46 79 4c b1 3d 3b 25 2f 25 c6 7f 25 0e 3f  >..FyL.=;%/%..%.?<
 0015b0 31 f7 9d ef 4b 33 30 a1 09 4a f3 80 b0 82 a2 10  >1...K30..J......<
 0015c0 f3 2a 0e 36 4f 75 c7 fe d1 b2 88 71 cb 5f 95 e8  >.*.6Ou.....q._..<
 0015d0 ee 38 fc 4f b3 7f f6 8f 8d 03 1f 8a 4f d5 bd fa  >.8.O........O...<
 0015e0 0f 8d 46 8c 1d c7 9a 2e 44 79 99 36 49 51 42 15  >..F.....Dy.6IQB.<
 0015f0 65 80 3b d2 7f da f5 1e 01 06 8d 94 43 8e 5a fc  >e.;.........C.Z.<
 001600 af 43 ab 4e 07 1b 4a c0 36 50 04 5e 8a d7 13 d5  >.C.N..J.6P.^....<
 001610 19 fc 09 f0 a2 96 78 9b 0a 52 72 82 93 fb 46 26  >......x..Rr...F&<
 001620 0b c1 91 ee d5 e1 4a 8d b7 ad 61 20 73 4d 73 7d  >......J...a sMs}<
 001630 4a 47 bb 57 85 2f e1 2a 43 78 b4 47 5d 0d a9 a4  >JG.W./.*Cx.G]...<
 001640 3c 85 2c 3a 2e 82 02 85 f5 0e d1 4c 51 c0 cb e9  ><.,:.......LQ...<
 001650 07 76 19 17 46 59 72 16 4f 67 0d c3 13 19 0e cb  >.v..FYr.Og......<
 001660 2e a9 53 1e 77 50 4b 45 69 41 d2 9d ac 55 a4 27  >..S.wPKEiA...U.'<
 001670 f4 3d f4 e4 33 d6 06 e6 21 e6 8f ce 69 96 d1 a0  >.=..3...!...i...<
 001680 87 09 36 5a 8d fa b7 e0 5b aa 6f c1 bd 56 0c f9  >..6Z....[.o..V..<
 001690 32 cc b3 df 97 d3 62 42 2c 55 be b5 00 f3 aa f4  >2.....bB,U......<
 0016a0 87 51 01 5a 47 7e fb 9e c3 56 06 0b e4 fa 3c 2c  >.Q.ZG~...V....<,<
 0016b0 ba fe 0f 33 14 76 54 77 47 5d 3d 50 01 e6 e3 6b  >...3.vTwG]=P...k<
 0016c0 8d ec 79 3c 0a b3 91 9f 33 9c 30 b8 39 8f 07 51  >..y<....3.0.9..Q<
 0016d0 7d 93 d2 3a 14 98 8d 80 d8 71 c3 62 45 94 a4 92  >}..:.....q.bE...<
 0016e0 8d 56 bf 78 02 e4 5f 61 42 a5 f9 f8 16 22 e3 73  >.V.x.._aB....".s<
 0016f0 70 cb 32 e8 2d bf 15 e0 a2 87 c0 b0 51 b9 26 e7  >p.2.-.......Q.&.<
 001700 50 be a0 76 57 16 e2 ae e8 f8 06 1e a8 ef 61 ec  >P..vW.........a.<
 001710 63 0f a7 a5 50 09 75 12 52 a5 a5 29 1a 74 5b 71  >c...P.u.R..).t[q<
 001720 6b 7b 39 bd 57 59 af c9 d4 ec 3d f9 33 51 31 c9  >k{9.WY....=.3Q1.<
 001730 6a 75 cf 42 8e 85 6a 71 64 dc ff 00 35 2a 07 60  >ju.B..jqd...5*.`<
 001740 37 d8 6f bd b8 a9 40 26 be 11 85 62 51 71 0c 35  >7.o...@&...bQq.5<
 001750 c5 ad 82 43 ac 95 8b 14 90 45 d0 ae c2 47 06 dc  >...C.....E...G..<
 001760 82 0e d7 b5 7a c7 a1 21 2b 8f 89 c6 61 2d 42 9e  >....z..!+...a-B.<
 001770 92 e3 68 4f 08 50 36 5a 7d 96 3b 8f 61 14 ed 91  >..hO.P6Z}.;.a...<
 001780 32 ce 36 25 05 cb 8c 96 21 25 62 42 91 21 23 5b  >2.6%....!%bB.!#[<
 001790 6a 49 b5 c2 4e e9 24 6a 17 3d 9b f7 1a 5c 92 e2  >jI..N.$j.=...\..<
 0017a0 26 e4 c7 1a eb 13 86 cd 50 69 4a 20 d9 b7 2d 64  >&.......PiJ ..-d<
 0017b0 dc 73 ba 49 f0 da 80 36 43 f5 46 7f 02 7c 28 ac  >.s.I...6C.F..|(.<
 0017c0 c4 1f 34 67 f0 27 c2 8a 45 e2 74 7a 0b a5 1c 91  >..4g.'..E.tz....<
 0017d0 e6 6f a9 48 f7 6a f0 af 39 1e 1c 09 98 d3 26 7a  >.o.H.j..9.....&z<
 0017e0 ed 1e 38 5c 87 75 24 10 52 84 95 71 7d f8 ee 3b  >..8\.u$.R..q}..;<
 0017f0 0e ea f5 37 d4 64 7b b5 78 52 dc 39 92 70 e9 09  >...7.d{.xR.9.p..<
 001800 91 1d c5 b2 fa 2c a4 2d 26 c4 53 34 70 33 3a 43  >.....,.-&.S4p3:C<
 001810 dd 86 45 97 2b 3b 39 89 2b cc 1f 9e fb a9 2f 28  >..E.+;9.+...../(<
 001820 c7 96 ca 84 54 b4 a0 2c 8d 2a 55 88 16 51 d5 ab  >....T..,.*U..Q..<
 001830 bc 5b 8b 96 dc 9a fc e9 4e 4d 4e 23 2c 88 6d 25  >.[......NMN#,.m%<
 001840 0d b6 8e 97 52 d6 52 41 d6 56 0f d1 23 8b 1e b0  >....R.RA.V..#...<
 001850 3b dc 5a aa 48 99 de 7b 6c 2a 24 c6 22 4b 86 a5  >;.Z.H..{l*$."K..<
 001860 85 74 2e b0 9d 28 3c 5d 20 5a c6 9b 70 5c cf 96  >.t...(<] Z..p\..<
 001870 a1 c3 e9 e3 6b 8d 25 e5 2b a7 68 a8 a1 b4 ec 2d  >....k.%.+.h....-<
 001880 a1 20 28 da f7 ed e6 fd 96 02 d3 3c 38 e1 19 7f  >. (........<8...<
 001890 03 c2 e4 cb c4 e3 c2 5b 09 64 02 d9 7d 61 7a 6d  >.......[.d..}azm<
 0018a0 b6 a0 35 6c 38 55 d4 01 1b ef d8 25 f1 f9 cb 63  >..5l8U.....%...c<
 0018b0 03 6c 27 0a 9d 25 4f a9 2d a9 a8 6b 29 5b 7a b7  >.l'..%O.-..k)[z.<
 0018c0 27 5a 4e c3 62 36 da f6 1c 6f 55 6a 73 fc 54 ad  >'ZN.b6...oUjs.T.<
 0018d0 d6 99 cb f1 17 28 05 06 64 a0 85 75 b7 d2 ae b2  >.....(..d..u....<
 0018e0 47 1b 7f d5 6c cc b9 c7 33 ba db 6c ab 0c 97 15  >G...l...3..l....<
 0018f0 87 2c e6 b6 92 a4 77 1b 21 49 d8 8b 0d c2 b5 6e  >.,....w.!I.....n<
 001900 4e c2 c2 c5 c0 3c c3 c1 9b 83 85 c9 87 83 35 0b  >N....<........5.<
 001910 ce 5f 42 48 62 5b 36 53 68 22 c4 3a 46 ea 3f 4b  >._BHb[6Sh".:F.?K<
 001920 9a 41 cf 98 03 39 47 2c e1 98 5a 14 97 24 4c 7d  >.A...9G,..Z..$L}<
 001930 72 5f 71 1b 24 14 80 02 52 0e f6 eb ed 7e ef 6d  >r_q.$...R....~.m<
 001940 29 bf 9b b3 04 b6 fd 3e 2b 20 a7 82 12 bd 24 f6  >)......>+ ....$.<
 001950 f6 56 8c 7f 30 cd cc 53 9b 93 33 42 7a 26 92 cb  >.V..0..S..3Bz&..<
 001960 4d b6 2c 94 20 76 0b ee 7b 77 34 12 4c c4 f5 46  >M.,. v..{w4.L..F<
 001970 7d da 7c 28 a2 1f a9 b3 ee d3 e1 45 23 2c 4e 8f  >}.|(.......E#,N.<
 001980 41 74 a3 92 33 2c e9 88 f1 ed 08 57 85 2a c9 7f  >At..3,.....W.*..<
 001990 a6 58 27 94 8b 71 d9 4e 8d e1 ee e2 8e a2 03 1a  >.X'..q.N........<
 0019a0 7a 69 44 32 8d 46 c3 52 b6 17 fd 6b 87 19 f2 6f  >ziD2.F.R...k...o<
 0019b0 99 30 89 8d c5 5c 2f 38 75 c4 29 68 11 ce bd 41  >.0...\/8u.)h...A<
 0019c0 22 e6 c3 9b 8e ee db ed 7a 62 8e 06 63 48 57 56  >".......zb..cHWV<
 0019d0 19 7f 45 2b d6 6f 62 2b ae 36 15 3a 62 14 a8 91  >..E+.ob+.6.:b...<
 0019e0 5e 7c 21 25 4b 0d 20 a8 a4 0e 49 b5 49 61 39 66  >^|!%K. ...I.Ia9f<
 0019f0 7c cc 45 98 8e c7 5b 0e 3d f5 7d 3a 4a 02 b9 e3  >|.E...[.=.}:J...<
 001a00 6b 9d c5 b6 06 ae 33 e4 8e 4f 6e 3b 53 9b c4 31  >k.....3..On;S..1<
 001a10 24 29 38 6b 0a 1d 22 f4 15 6a 55 89 4a 6d 63 70  >$)8k.."..jU.Jmcp<
 001a20 6d bf b0 1a 71 56 78 9d 1d 6e 33 39 f4 29 89 08  >m...qVx..n39.)..<
 001a30 2a 85 3e 3b 7d 52 7d a0 fb 79 1d 9f 98 34 b1 9a  >*.>;}R}..y...4..<
 001a40 5d c6 f2 ef f2 08 6d 4c e1 d6 0b 20 01 67 c9 db  >].....mL... .g..<
 001a50 52 8d ae 38 fa 24 ed 6e de 6a 27 0f 71 6c 41 e8  >R..8.$.n.j'.qlA.<
 001a60 b1 16 d4 ac 39 fb a9 16 55 f4 38 01 01 40 03 ed  >....9...U.8..@..<
 001a70 20 f6 6f ec a0 81 e2 44 77 b1 44 7c a7 8a e1 44  > .o....Dw.D|...D<
 001a80 cd 83 d5 79 45 36 44 96 4d c1 3c 6e 07 78 ec 24  >...yE6D.M.<n.x.$<
 001a90 f6 6d aa 57 92 07 31 2c 25 bc 53 2f 49 71 49 71  >.m.W..1,%.S/IqIq<
 001aa0 3a d1 16 62 42 54 46 fc 28 6d be d6 b8 02 c7 9a  >:..bBTF.(m......<
 001ab0 8e c0 33 b4 96 e0 31 19 fb 2f cd ae 1b 51 3d 6d  >..3...1../...Q=m<
 001ac0 86 d7 ff 00 11 57 f6 08 a5 7c 99 18 29 94 b4 a0  >.....W...|..)...<
 001ad0 80 92 da 6d 64 90 2d 61 6e eb 54 3e 44 a4 7c e6  >...md.-an.T>D.|.<
 001ae0 22 3f 07 e6 92 51 d1 be c7 a3 71 04 df 4a 93 b1  >"?...Q....q..J..<
 001af0 1b 7b 45 15 2f 99 37 cd 18 b5 ff 00 b6 bd fe b3  >.{E./.7.........<
 001b00 45 24 f1 3a 45 0e d4 72 47 14 69 0e c2 96 cc a8  >E$.:E..rG.i.....<
 001b10 eb d0 f3 2b 0e 36 ab 03 65 03 70 6c 76 e6 98 17  >...+.6..e.plv...<
 001b20 9f f3 3b 84 15 62 60 9b dc 7c dd ae 7f 6d 14 57  >..;..b`..|...m.W<
 001b30 ab da 0a 94 29 55 69 d4 8a 79 a4 c8 06 e6 49 62  >....)Ui..y....Ib<
 001b40 7c b9 cd 3a 5b 93 2d 45 4f 2d 00 0d 44 9b 92 2d  >|..:[.-EO-..D..-<
 001b50 b0 37 ee b5 6e 73 19 c4 dc 51 5a e7 c8 2b 20 02  >.7..ns...QZ..+ .<
 001b60 bd 67 51 03 8d f9 a2 8a 35 a5 e4 af 72 b3 7e 71  >.gQ.....5...r.~q<
 001b70 f4 8c 27 16 9a 88 32 a1 74 8d a9 99 43 d3 85 b2  >..'...2.t...C...<
 001b80 85 29 7f 7a 88 d5 f7 6f b7 22 a1 7e 4a 86 4f d4  >.).z...o.".~J.O.<
 001b90 ff 00 98 fc 68 a2 8d 69 79 0d ca cd f9 c7 d2 36  >....h..iy......6<
 001ba0 35 02 3b 2b d4 db 61 27 ef 26 9a 9b cf 19 91 a6  >5.;+..a'.&......<
 001bb0 f4 23 12 21 22 ff 00 d0 b7 ff 00 1a 28 a3 5a 5e  >.#.!".......(.Z^<
 001bc0 43 72 b3 7e 71 f4 88 39 12 1d 95 25 d9 2f af 5b  >Cr.~q..9...%./.[<
 001bd0 ce ac b8 b5 5a d7 51 37 27 f5 a2 8a 2a 06 12 b9  >....Z.Q7'...*...<
 001be0 5c 8f ff d9 55 46 49 44 00 00 00 3b 00 00 68 74  >\...UFID...;..ht<
 001bf0 74 70 3a 2f 2f 6d 75 73 69 63 62 72 61 69 6e 7a  >tp://musicbrainz<
 001c00 2e 6f 72 67 00 61 30 30 37 35 34 66 30 2d 66 34  >.org.a00754f0-f4<
 001c10 66 34 2d 34 65 37 66 2d 39 63 66 37 2d 37 32 38  >f4-4e7f-9cf7-728<
 001c20 30 30 66 62 39 61 34 37 34 54 53 4f 50 00 00 00  >00fb9a474TSOP...<
 001c30 27 00 00 01 ff fe 4c 00 65 00 66 00 74 00 20 00  >'.....L.e.f.t. .<
 001c40 57 00 69 00 6e 00 67 00 20 00 46 00 61 00 73 00  >W.i.n.g. .F.a.s.<
 001c50 63 00 69 00 73 00 74 00 73 00 54 58 58 58 00 00  >c.i.s.t.s.TXXX..<
 001c60 00 77 00 00 01 ff fe 4d 00 75 00 73 00 69 00 63  >.w.....M.u.s.i.c<
 001c70 00 42 00 72 00 61 00 69 00 6e 00 7a 00 20 00 41  >.B.r.a.i.n.z. .A<
 001c80 00 6c 00 62 00 75 00 6d 00 20 00 49 00 64 00 00  >.l.b.u.m. .I.d..<
 001c90 00 ff fe 66 00 61 00 30 00 63 00 35 00 33 00 35  >...f.a.0.c.5.3.5<
 001ca0 00 63 00 2d 00 38 00 62 00 66 00 61 00 2d 00 34  >.c.-.8.b.f.a.-.4<
 001cb0 00 37 00 39 00 35 00 2d 00 61 00 32 00 36 00 64  >.7.9.5.-.a.2.6.d<
 001cc0 00 2d 00 39 00 39 00 63 00 62 00 36 00 63 00 38  >.-.9.9.c.b.6.c.8<
 001cd0 00 34 00 39 00 37 00 65 00 37 00 54 50 55 42 00  >.4.9.7.e.7.TPUB.<
 001ce0 00 00 1b 00 00 01 ff fe 53 00 6b 00 79 00 72 00  >........S.k.y.r.<
 001cf0 61 00 74 00 20 00 4d 00 75 00 73 00 69 00 63 00  >a.t. .M.u.s.i.c.<
 001d00 54 58 58 58 00 00 00 4d 00 00 01 ff fe 4d 00 75  >TXXX...M.....M.u<
 001d10 00 73 00 69 00 63 00 42 00 72 00 61 00 69 00 6e  >.s.i.c.B.r.a.i.n<
 001d20 00 7a 00 20 00 41 00 6c 00 62 00 75 00 6d 00 20  >.z. .A.l.b.u.m. <
 001d30 00 52 00 65 00 6c 00 65 00 61 00 73 00 65 00 20  >.R.e.l.e.a.s.e. <
 001d40 00 43 00 6f 00 75 00 6e 00 74 00 72 00 79 00 00  >.C.o.u.n.t.r.y..<
 001d50 00 ff fe 55 00 53 00 54 58 58 58 00 00 00 47 00  >...U.S.TXXX...G.<
 001d60 00 01 ff fe 4d 00 75 00 73 00 69 00 63 00 42 00  >....M.u.s.i.c.B.<
 001d70 72 00 61 00 69 00 6e 00 7a 00 20 00 41 00 6c 00  >r.a.i.n.z. .A.l.<
 001d80 62 00 75 00 6d 00 20 00 53 00 74 00 61 00 74 00  >b.u.m. .S.t.a.t.<
 001d90 75 00 73 00 00 00 ff fe 6f 00 66 00 66 00 69 00  >u.s.....o.f.f.i.<
 001da0 63 00 69 00 61 00 6c 00 00 00 00 00 00 00 00 00  >c.i.a.l.........<
 001db0 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00  >................<
 *
 001f70 00 ff fb 90 64 00 0f f0 00 00 00 00 00 00 00 00  >....d...........<
 001f80 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00  >................<
 *
 002110 00 00 ff fb 92 64 be 8f f0 00 00 00 00 00 00 00  >.....d..........<
 002120 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00  >................<
 *
 0022b0 00 00 00 00 ff fb 92 64 ff 8f f0 00 00 00 00 00  >.......d........<
 0022c0 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00  >................<
 *
 002450 00 00 00 00 00 00 ff fb 92 64 ff 8f f0 00 00 00  >.........d......<
 002460 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00  >................<
 *
 0025f0 00 00 00 00 00 00 00 00 ff fb 92 64 ff 8f f0 00  >...........d....<
 002600 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00  >................<
 *
 002790 00 00 00 00 00 00 00 00 00 00                    >..........<
 00279a
 \endcode
 *
 * Second tag, broken out:
 *
 \code
 000f67                      49 44 33 04 00              ID3v2.4
 000f6c                                     00           no flags
 000f6d                                        00 00 20  0x1000 = 4096 bytes
 000f70 00
 000f71    54 49 54 32 00 00 00 31 00 00                 TIT2 0x31 = 49 bytes, no flags
 000f7b                                  01              UTF-16 encoding
 000f7c                                     ff fe 4a 00  >..J.<
 000f80 6f 00 20 00 4f 00 6e 00 65 00 73 00 20 00 55 00  >o. .O.n.e.s. .U.<
 000f90 67 00 6c 00 79 00 20 00 41 00 66 00 74 00 65 00  >g.l.y. .A.f.t.e.<
 000fa0 72 00 20 00 32 00 20 00 41 00 4d 00
 000fac                                     54 50 45 31  TPE1 0x27=39 bytes, no flags
 000fb0 00 00 00 27 00 00
 000fb5                   01                             UTF-16 encoded
 000fb6                      ff fe 4c 00 65 00 66 00 74  >...'.....L.e.f.t<
 000fc0 00 20 00 57 00 69 00 6e 00 67 00 20 00 46 00 61  >. .W.i.n.g. .F.a<
 000fd0 00 73 00 63 00 69 00 73 00 74 00 73 00
 000fdd                                        54 52 43  TRCK, 0x05 bytes, no flags
 000fe0 4b 00 00 00 05 00 00
 000fe7                      00                          ISO-8859-1 encoding
 000fe8                         37 2f 31 31              "7/11"
 000fec                                     54 41 4c 42  TALB, 0x1b = 27 bytes, no flags, UTF-16 encoded
 000ff0 00 00 00 1b 00 00 01
 000ff7                      ff fe 41 00 6c 00 6c 00 20  >.........A.l.l. <
 001000 00 46 00 69 00 72 00 65 00 64 00 20 00 55 00 70  >.F.i.r.e.d. .U.p<
 001010 00
 001011    54 50 4f 53 00 00 00 04 00 00                 TPOS, 0x04 bytes, no flags
 00101b                                  00 31 2f 31     ISO-8859 "1/1"
 00101f                                              54  TDRC, 0x17 = 23 bytes, no flags
 001020 44 52 43 00 00 00 17 00 00
 001029                            01 ff fe 31 00 39 00  >...1.9.<
 001030 39 00 31 00 2d 00 30 00 31 00 2d 00 30 00 31 00  >9.1.-.0.1.-.0.1.<
 001040 54 43 4f 4e 00 00 00 0b 00 00                    TCON, 0x0b = 11 bytes, no flags
 00104a                               01 ff fe 50 00 75  >...P.u<
 001050 00 6e 00 6b 00
 001055                54 4d 45 44 00 00 00 1d 00 00     TMED, 0x1d = 29 bytes, no flags
 00105f                                              01  UTF-16 encoding
 001060 ff fe 44 00 69 00 67 00 69 00 74 00 61 00 6c 00  >..D.i.g.i.t.a.l.<
 001070 20 00 4d 00 65 00 64 00 69 00 61 00
 00107c                                     54 58 58 58  TXXX, 0x1b = 27 bytes, no flags
 001086                   01 ff fe 53 00 43 00 52 00 49  >.........S.C.R.I<
 001090 00 50 00 54 00 00 00
 001090 00 50 00 54 00 00 00 ff fe 4c 00 61 00 74 00 6e  >.P.T.....L.a.t.n<
 0010a0 00
 0010a1    54 58 58 58 00 00 00 65 00 00                 TXXX, 0x65 = 101 bytes, no flags
 0010ab                                  01 ff fe 41 00  >...A.<
 0010b0 63 00 6f 00 75 00 73 00 74 00 69 00 64 00 20 00  >c.o.u.s.t.i.d. .<
 0010c0 49 00 64 00 00 00 ff fe 34 00 35 00 62 00 65 00  >I.d.....4.5.b.e.<
 0010d0 33 00 33 00 36 00 65 00 2d 00 35 00 31 00 64 00  >3.3.6.e.-.5.1.d.<
 0010e0 61 00 2d 00 34 00 66 00 62 00 38 00 2d 00 39 00  >a.-.4.f.b.8.-.9.<
 0010f0 64 00 35 00 64 00 2d 00 31 00 31 00 63 00 35 00  >d.5.d.-.1.1.c.5.<
 001100 32 00 64 00 37 00 64 00 38 00 30 00 61 00 31 00  >2.d.7.d.8.0.a.1.<
 001110 54 58 58 58 00 00 00 3d 00 00                    TXXX, 0x3d = 61 bytes, no flags
 00111a                               01 ff fe 4d 00 75  >...M.u<
 001120 00 73 00 69 00 63 00 42 00 72 00 61 00 69 00 6e  >.s.i.c.B.r.a.i.n<
 001130 00 7a 00 20 00 41 00 6c 00 62 00 75 00 6d 00 20  >.z. .A.l.b.u.m. <
 001140 00 54 00 79 00 70 00 65 00 00 00 ff fe 61 00 6c  >.T.y.p.e.....a.l<
 001150 00 62 00 75 00 6d 00
 001157                      54 58 58 58 00 00 01 05 00  TXXX, $0105 => $85 = 133 bytes, no flags
 001160 00
 001161    01 ff fe 4d 00 75 00 73 00 69 00 63 00 42 00  >...M.u.s.i.c.B.<
 001170 72 00 61 00 69 00 6e 00 7a 00 20 00 41 00 6c 00  >r.a.i.n.z. .A.l.<
 001180 62 00 75 00 6d 00 20 00 41 00 72 00 74 00 69 00  >b.u.m. .A.r.t.i.<
 001190 73 00 74 00 20 00 49 00 64 00 00 00 ff fe 31 00  >s.t. .I.d.....1.<
 0011a0 66 00 63 00 35 00 35 00 62 00 37 00 62 00 2d 00  >f.c.5.5.b.7.b.-.<
 0011b0 64 00 65 00 63 00 31 00 2d 00 34 00 38 00 30 00  >d.e.c.1.-.4.8.0.<
 0011c0 66 00 2d 00 38 00 35 00 39 00 39 00 2d 00 35 00  >f.-.8.5.9.9.-.5.<
 0011d0 63 00 31 00 61 00 63 00 38 00 33 00 31 00 33 00  >c.1.a.c.8.3.1.3.<
 0011e0 63 00 30 00 36 00                                >c.0.6.<
 0011e6                   54 58 58 58 00 00 00 79 00 00  TXXX, 0x79 = 121 bytes, no flags
 0011f0 01 ff fe 4d 00 75 00 73 00 69 00 63 00 42 00 72  >...M.u.s.i.c.B.r<
 001200 00 61 00 69 00 6e 00 7a 00 20 00 41 00 72 00 74  >.a.i.n.z. .A.r.t<
 001210 00 69 00 73 00 74 00 20 00 49 00 64 00 00 00 ff  >.i.s.t. .I.d....<
 001220 fe 31 00 66 00 63 00 35 00 35 00 62 00 37 00 62  >.1.f.c.5.5.b.7.b<
 001230 00 2d 00 64 00 65 00 63 00 31 00 2d 00 34 00 38  >.-.d.e.c.1.-.4.8<
 001240 00 30 00 66 00 2d 00 38 00 35 00 39 00 39 00 2d  >.0.f.-.8.5.9.9.-<
 001250 00 35 00 63 00 31 00 61 00 63 00 38 00 33 00 31  >.5.c.1.a.c.8.3.1<
 001260 00 33 00 63 00 30 00 36 00                       >.3.c.0.6.
 001269                            54 44 4f 52 00 00 00  TDOR, 0x17 = 23 bytes, no flags
 001270 17 00 00
 001273          01 ff fe 31 00 39 00 39 00 31 00 2d 00  >......1.9.9.1.-.<
 001280 30 00 31 00 2d 00 30 00 31 00                    >0.1.-.0.1.<
 00128a                               54 53 4f 32 00 00  TSO2, 0x27 = 39 bytes, no flags
 001290 00 27 00 00
 001294             01 ff fe 4c 00 65 00 66 00 74 00 20  >.'.....L.e.f.t. <
 0012a0 00 57 00 69 00 6e 00 67 00 20 00 46 00 61 00 73  >.W.i.n.g. .F.a.s<
 0012b0 00 63 00 69 00 73 00 74 00 73 00                 >.c.i.s.t.s.<
 0012bb                                  54 50 45 32 00  TPE2, 0x27 = 39 bytes, no flags
 0012c0 00 00 27 00 00
 0012c6                01 ff fe 4c 00 65 00 66 00 74 00  >..'.....L.e.f.t.<
 0012d0 20 00 57 00 69 00 6e 00 67 00 20 00 46 00 61 00  > .W.i.n.g. .F.a.<
 0012e0 73 00 63 00 69 00 73 00 74 00 73 00              >s.c.i.s.t.s.<
 0012ec                                     54 58 58 58  TXXX, $0107->0x87 = 135 bytes, no flags
 0012f6                   01 ff fe 4d 00 75 00 73 00 69  >...M.u.s.i<
 001300 00 63 00 42 00 72 00 61 00 69 00 6e 00 7a 00 20  >.c.B.r.a.i.n.z. <
 001310 00 52 00 65 00 6c 00 65 00 61 00 73 00 65 00 20  >.R.e.l.e.a.s.e. <
 001320 00 47 00 72 00 6f 00 75 00 70 00 20 00 49 00 64  >.G.r.o.u.p. .I.d<
 001330 00 00 00 ff fe 31 00 62 00 34 00 66 00 62 00 38  >.....1.b.4.f.b.8<
 001340 00 64 00 66 00 2d 00 61 00 33 00 63 00 61 00 2d  >.d.f.-.a.3.c.a.-<
 001350 00 34 00 64 00 30 00 30 00 2d 00 62 00 32 00 32  >.4.d.0.0.-.b.2.2<
 001360 00 62 00 2d 00 38 00 61 00 35 00 61 00 32 00 32  >.b.-.8.a.5.a.2.2<
 001370 00 38 00 32 00 62 00 31 00 35 00 30 00           >.8.2.b.1.5.0.<
 00137d                                        41 50 49  APIC, $105d->0x85d = 2141 bytes, no flags
 001380 43 00 00 10 5d 00 00
 001387                      00 69 6d 61 67 65 2f 6a 70  >.image/jp<
 001390 65 67 00 03 00 ff d8 ff e0 00 10 4a 46 49 46 00  >eg.........JFIF.<
 0013a0 01 01 01 00 c8 00 c8 00 00 ff db 00 43 00 08 06  >............C...<
 0013b0 06 07 06 05 08 07 07 07 09 09 08 0a 0c 14 0d 0c  >................<
 0013c0 0b 0b 0c 19 12 13 0f 14 1d 1a 1f 1e 1d 1a 1c 1c  >................<
 0013d0 20 24 2e 27 20 22 2c 23 1c 1c 28 37 29 2c 30 31  > $.' ",#..(7),01<
 0013e0 34 34 34 1f 27 39 3d 38 32 3c 2e 33 34 32 ff db  >444.'9=82<.342..<
 0013f0 00 43 01 09 09 09 0c 0b 0c 18 0d 0d 18 32 21 1c  >.C...........2!.<
 001400 21 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32  >!222222222222222<
 001410 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32  >2222222222222222<
 *
 001430 32 32 32 ff c0 00 11 08 00 46 00 46 03 01 22 00  >222......F.F..".<
 001440 02 11 01 03 11 01 ff c4 00 1c 00 00 02 02 03 01  >................<
 001450 01 00 00 00 00 00 00 00 00 00 00 00 06 05 07 01  >................<
 001460 03 04 02 08 ff c4 00 42 10 00 01 03 02 05 00 05  >.......B........<
 001470 07 0a 01 0d 00 00 00 00 00 01 02 03 04 00 11 05  >................<
 001480 06 12 21 31 13 22 41 51 b1 07 14 23 34 61 73 d1  >..!1."AQ...#4as.<
 001490 15 24 32 33 53 71 72 81 91 92 16 25 42 43 52 54  >.$23Sqr....%BCRT<
 0014a0 62 a1 a2 b2 c1 d2 e1 f1 ff c4 00 1b 01 00 02 03  >b...............<
 0014b0 01 01 01 00 00 00 00 00 00 00 00 00 00 00 04 01  >................<
 0014c0 03 06 07 02 05 ff c4 00 29 11 00 02 01 01 04 0a  >........).......<
 0014d0 03 01 00 00 00 00 00 00 00 00 00 01 02 03 04 11  >................<
 0014e0 31 71 06 12 13 14 15 21 23 33 51 91 05 41 53 c1  >1q.....!#3Q..AS.<
 0014f0 ff da 00 0c 03 01 00 02 11 03 11 00 3f 00 41 79  >............?.Ay<
 001500 e4 b0 ca dd 50 25 29 17 36 a8 df e2 08 9f 66 f7  >....P%).6.....f.<
 001510 e8 3e 35 d7 88 6f 01 f1 fd c3 e1 4a 20 52 f4 e0  >.>5..o.....J R..<
 001520 a4 b9 9a ef 96 f9 0a f6 5a 91 8d 3b ae 68 64 4e  >........Z..;.hdN<
 001530 3b 19 76 b3 4f 5c f1 d5 1f 1a da ac 55 84 b7 ac  >;.v.O\......U...<
 001540 b6 ed b8 fa 23 63 dd cd 71 65 ec 1a 66 24 fa d5  >....#c..qe..f$..<
 001550 11 29 50 68 5d 6a 52 4a 85 c8 3a 45 85 c9 37 07  >.)Ph]jRJ..:E..7.<
 001560 60 3b 29 8a 36 57 c6 e2 b0 e2 bc c2 78 29 58 48  >`;).6W......x)XH<
 001570 52 90 b4 a5 63 fa a1 00 15 28 71 b9 b0 da ac d9  >R...c....(q.....<
 001580 44 f9 5c 72 d7 e5 7a 21 55 8d 47 46 ea 69 e0 38  >D.\r..z!U.GF.i.8<
 001590 e0 7c 6b cf cb d1 7e cd ef da 3e 35 30 ac 1b 0e  >.|k...~...>50...<
 0015a0 92 a5 46 79 4c b1 3d 3b 25 2f 25 c6 7f 25 0e 3f  >..FyL.=;%/%..%.?<
 0015b0 31 f7 9d ef 4b 33 30 a1 09 4a f3 80 b0 82 a2 10  >1...K30..J......<
 0015c0 f3 2a 0e 36 4f 75 c7 fe d1 b2 88 71 cb 5f 95 e8  >.*.6Ou.....q._..<
 0015d0 ee 38 fc 4f b3 7f f6 8f 8d 03 1f 8a 4f d5 bd fa  >.8.O........O...<
 0015e0 0f 8d 46 8c 1d c7 9a 2e 44 79 99 36 49 51 42 15  >..F.....Dy.6IQB.<
 0015f0 65 80 3b d2 7f da f5 1e 01 06 8d 94 43 8e 5a fc  >e.;.........C.Z.<
 001600 af 43 ab 4e 07 1b 4a c0 36 50 04 5e 8a d7 13 d5  >.C.N..J.6P.^....<
 001610 19 fc 09 f0 a2 96 78 9b 0a 52 72 82 93 fb 46 26  >......x..Rr...F&<
 001620 0b c1 91 ee d5 e1 4a 8d b7 ad 61 20 73 4d 73 7d  >......J...a sMs}<
 001630 4a 47 bb 57 85 2f e1 2a 43 78 b4 47 5d 0d a9 a4  >JG.W./.*Cx.G]...<
 001640 3c 85 2c 3a 2e 82 02 85 f5 0e d1 4c 51 c0 cb e9  ><.,:.......LQ...<
 001650 07 76 19 17 46 59 72 16 4f 67 0d c3 13 19 0e cb  >.v..FYr.Og......<
 001660 2e a9 53 1e 77 50 4b 45 69 41 d2 9d ac 55 a4 27  >..S.wPKEiA...U.'<
 001670 f4 3d f4 e4 33 d6 06 e6 21 e6 8f ce 69 96 d1 a0  >.=..3...!...i...<
 001680 87 09 36 5a 8d fa b7 e0 5b aa 6f c1 bd 56 0c f9  >..6Z....[.o..V..<
 001690 32 cc b3 df 97 d3 62 42 2c 55 be b5 00 f3 aa f4  >2.....bB,U......<
 0016a0 87 51 01 5a 47 7e fb 9e c3 56 06 0b e4 fa 3c 2c  >.Q.ZG~...V....<,<
 0016b0 ba fe 0f 33 14 76 54 77 47 5d 3d 50 01 e6 e3 6b  >...3.vTwG]=P...k<
 0016c0 8d ec 79 3c 0a b3 91 9f 33 9c 30 b8 39 8f 07 51  >..y<....3.0.9..Q<
 0016d0 7d 93 d2 3a 14 98 8d 80 d8 71 c3 62 45 94 a4 92  >}..:.....q.bE...<
 0016e0 8d 56 bf 78 02 e4 5f 61 42 a5 f9 f8 16 22 e3 73  >.V.x.._aB....".s<
 0016f0 70 cb 32 e8 2d bf 15 e0 a2 87 c0 b0 51 b9 26 e7  >p.2.-.......Q.&.<
 001700 50 be a0 76 57 16 e2 ae e8 f8 06 1e a8 ef 61 ec  >P..vW.........a.<
 001710 63 0f a7 a5 50 09 75 12 52 a5 a5 29 1a 74 5b 71  >c...P.u.R..).t[q<
 001720 6b 7b 39 bd 57 59 af c9 d4 ec 3d f9 33 51 31 c9  >k{9.WY....=.3Q1.<
 001730 6a 75 cf 42 8e 85 6a 71 64 dc ff 00 35 2a 07 60  >ju.B..jqd...5*.`<
 001740 37 d8 6f bd b8 a9 40 26 be 11 85 62 51 71 0c 35  >7.o...@&...bQq.5<
 001750 c5 ad 82 43 ac 95 8b 14 90 45 d0 ae c2 47 06 dc  >...C.....E...G..<
 001760 82 0e d7 b5 7a c7 a1 21 2b 8f 89 c6 61 2d 42 9e  >....z..!+...a-B.<
 001770 92 e3 68 4f 08 50 36 5a 7d 96 3b 8f 61 14 ed 91  >..hO.P6Z}.;.a...<
 001780 32 ce 36 25 05 cb 8c 96 21 25 62 42 91 21 23 5b  >2.6%....!%bB.!#[<
 001790 6a 49 b5 c2 4e e9 24 6a 17 3d 9b f7 1a 5c 92 e2  >jI..N.$j.=...\..<
 0017a0 26 e4 c7 1a eb 13 86 cd 50 69 4a 20 d9 b7 2d 64  >&.......PiJ ..-d<
 0017b0 dc 73 ba 49 f0 da 80 36 43 f5 46 7f 02 7c 28 ac  >.s.I...6C.F..|(.<
 0017c0 c4 1f 34 67 f0 27 c2 8a 45 e2 74 7a 0b a5 1c 91  >..4g.'..E.tz....<
 0017d0 e6 6f a9 48 f7 6a f0 af 39 1e 1c 09 98 d3 26 7a  >.o.H.j..9.....&z<
 0017e0 ed 1e 38 5c 87 75 24 10 52 84 95 71 7d f8 ee 3b  >..8\.u$.R..q}..;<
 0017f0 0e ea f5 37 d4 64 7b b5 78 52 dc 39 92 70 e9 09  >...7.d{.xR.9.p..<
 001800 91 1d c5 b2 fa 2c a4 2d 26 c4 53 34 70 33 3a 43  >.....,.-&.S4p3:C<
 001810 dd 86 45 97 2b 3b 39 89 2b cc 1f 9e fb a9 2f 28  >..E.+;9.+...../(<
 001820 c7 96 ca 84 54 b4 a0 2c 8d 2a 55 88 16 51 d5 ab  >....T..,.*U..Q..<
 001830 bc 5b 8b 96 dc 9a fc e9 4e 4d 4e 23 2c 88 6d 25  >.[......NMN#,.m%<
 001840 0d b6 8e 97 52 d6 52 41 d6 56 0f d1 23 8b 1e b0  >....R.RA.V..#...<
 001850 3b dc 5a aa 48 99 de 7b 6c 2a 24 c6 22 4b 86 a5  >;.Z.H..{l*$."K..<
 001860 85 74 2e b0 9d 28 3c 5d 20 5a c6 9b 70 5c cf 96  >.t...(<] Z..p\..<
 001870 a1 c3 e9 e3 6b 8d 25 e5 2b a7 68 a8 a1 b4 ec 2d  >....k.%.+.h....-<
 001880 a1 20 28 da f7 ed e6 fd 96 02 d3 3c 38 e1 19 7f  >. (........<8...<
 001890 03 c2 e4 cb c4 e3 c2 5b 09 64 02 d9 7d 61 7a 6d  >.......[.d..}azm<
 0018a0 b6 a0 35 6c 38 55 d4 01 1b ef d8 25 f1 f9 cb 63  >..5l8U.....%...c<
 0018b0 03 6c 27 0a 9d 25 4f a9 2d a9 a8 6b 29 5b 7a b7  >.l'..%O.-..k)[z.<
 0018c0 27 5a 4e c3 62 36 da f6 1c 6f 55 6a 73 fc 54 ad  >'ZN.b6...oUjs.T.<
 0018d0 d6 99 cb f1 17 28 05 06 64 a0 85 75 b7 d2 ae b2  >.....(..d..u....<
 0018e0 47 1b 7f d5 6c cc b9 c7 33 ba db 6c ab 0c 97 15  >G...l...3..l....<
 0018f0 87 2c e6 b6 92 a4 77 1b 21 49 d8 8b 0d c2 b5 6e  >.,....w.!I.....n<
 001900 4e c2 c2 c5 c0 3c c3 c1 9b 83 85 c9 87 83 35 0b  >N....<........5.<
 001910 ce 5f 42 48 62 5b 36 53 68 22 c4 3a 46 ea 3f 4b  >._BHb[6Sh".:F.?K<
 001920 9a 41 cf 98 03 39 47 2c e1 98 5a 14 97 24 4c 7d  >.A...9G,..Z..$L}<
 001930 72 5f 71 1b 24 14 80 02 52 0e f6 eb ed 7e ef 6d  >r_q.$...R....~.m<
 001940 29 bf 9b b3 04 b6 fd 3e 2b 20 a7 82 12 bd 24 f6  >)......>+ ....$.<
 001950 f6 56 8c 7f 30 cd cc 53 9b 93 33 42 7a 26 92 cb  >.V..0..S..3Bz&..<
 001960 4d b6 2c 94 20 76 0b ee 7b 77 34 12 4c c4 f5 46  >M.,. v..{w4.L..F<
 001970 7d da 7c 28 a2 1f a9 b3 ee d3 e1 45 23 2c 4e 8f  >}.|(.......E#,N.<
 001980 41 74 a3 92 33 2c e9 88 f1 ed 08 57 85 2a c9 7f  >At..3,.....W.*..<
 001990 a6 58 27 94 8b 71 d9 4e 8d e1 ee e2 8e a2 03 1a  >.X'..q.N........<
 0019a0 7a 69 44 32 8d 46 c3 52 b6 17 fd 6b 87 19 f2 6f  >ziD2.F.R...k...o<
 0019b0 99 30 89 8d c5 5c 2f 38 75 c4 29 68 11 ce bd 41  >.0...\/8u.)h...A<
 0019c0 22 e6 c3 9b 8e ee db ed 7a 62 8e 06 63 48 57 56  >".......zb..cHWV<
 0019d0 19 7f 45 2b d6 6f 62 2b ae 36 15 3a 62 14 a8 91  >..E+.ob+.6.:b...<
 0019e0 5e 7c 21 25 4b 0d 20 a8 a4 0e 49 b5 49 61 39 66  >^|!%K. ...I.Ia9f<
 0019f0 7c cc 45 98 8e c7 5b 0e 3d f5 7d 3a 4a 02 b9 e3  >|.E...[.=.}:J...<
 001a00 6b 9d c5 b6 06 ae 33 e4 8e 4f 6e 3b 53 9b c4 31  >k.....3..On;S..1<
 001a10 24 29 38 6b 0a 1d 22 f4 15 6a 55 89 4a 6d 63 70  >$)8k.."..jU.Jmcp<
 001a20 6d bf b0 1a 71 56 78 9d 1d 6e 33 39 f4 29 89 08  >m...qVx..n39.)..<
 001a30 2a 85 3e 3b 7d 52 7d a0 fb 79 1d 9f 98 34 b1 9a  >*.>;}R}..y...4..<
 001a40 5d c6 f2 ef f2 08 6d 4c e1 d6 0b 20 01 67 c9 db  >].....mL... .g..<
 001a50 52 8d ae 38 fa 24 ed 6e de 6a 27 0f 71 6c 41 e8  >R..8.$.n.j'.qlA.<
 001a60 b1 16 d4 ac 39 fb a9 16 55 f4 38 01 01 40 03 ed  >....9...U.8..@..<
 001a70 20 f6 6f ec a0 81 e2 44 77 b1 44 7c a7 8a e1 44  > .o....Dw.D|...D<
 001a80 cd 83 d5 79 45 36 44 96 4d c1 3c 6e 07 78 ec 24  >...yE6D.M.<n.x.$<
 001a90 f6 6d aa 57 92 07 31 2c 25 bc 53 2f 49 71 49 71  >.m.W..1,%.S/IqIq<
 001aa0 3a d1 16 62 42 54 46 fc 28 6d be d6 b8 02 c7 9a  >:..bBTF.(m......<
 001ab0 8e c0 33 b4 96 e0 31 19 fb 2f cd ae 1b 51 3d 6d  >..3...1../...Q=m<
 001ac0 86 d7 ff 00 11 57 f6 08 a5 7c 99 18 29 94 b4 a0  >.....W...|..)...<
 001ad0 80 92 da 6d 64 90 2d 61 6e eb 54 3e 44 a4 7c e6  >...md.-an.T>D.|.<
 001ae0 22 3f 07 e6 92 51 d1 be c7 a3 71 04 df 4a 93 b1  >"?...Q....q..J..<
 001af0 1b 7b 45 15 2f 99 37 cd 18 b5 ff 00 b6 bd fe b3  >.{E./.7.........<
 001b00 45 24 f1 3a 45 0e d4 72 47 14 69 0e c2 96 cc a8  >E$.:E..rG.i.....<
 001b10 eb d0 f3 2b 0e 36 ab 03 65 03 70 6c 76 e6 98 17  >...+.6..e.plv...<
 001b20 9f f3 3b 84 15 62 60 9b dc 7c dd ae 7f 6d 14 57  >..;..b`..|...m.W<
 001b30 ab da 0a 94 29 55 69 d4 8a 79 a4 c8 06 e6 49 62  >....)Ui..y....Ib<
 001b40 7c b9 cd 3a 5b 93 2d 45 4f 2d 00 0d 44 9b 92 2d  >|..:[.-EO-..D..-<
 001b50 b0 37 ee b5 6e 73 19 c4 dc 51 5a e7 c8 2b 20 02  >.7..ns...QZ..+ .<
 001b60 bd 67 51 03 8d f9 a2 8a 35 a5 e4 af 72 b3 7e 71  >.gQ.....5...r.~q<
 001b70 f4 8c 27 16 9a 88 32 a1 74 8d a9 99 43 d3 85 b2  >..'...2.t...C...<
 001b80 85 29 7f 7a 88 d5 f7 6f b7 22 a1 7e 4a 86 4f d4  >.).z...o.".~J.O.<
 001b90 ff 00 98 fc 68 a2 8d 69 79 0d ca cd f9 c7 d2 36  >....h..iy......6<
 001ba0 35 02 3b 2b d4 db 61 27 ef 26 9a 9b cf 19 91 a6  >5.;+..a'.&......<
 001bb0 f4 23 12 21 22 ff 00 d0 b7 ff 00 1a 28 a3 5a 5e  >.#.!".......(.Z^<
 001bc0 43 72 b3 7e 71 f4 88 39 12 1d 95 25 d9 2f af 5b  >Cr.~q..9...%./.[<
 001bd0 ce ac b8 b5 5a d7 51 37 27 f5 a2 8a 2a 06 12 b9  >....Z.Q7'...*...<
 001be0 5c 8f ff d9                                      >\...<
 001be4             55 46 49 44 00 00 00 3b 00 00        UFID, 0x3b = 59 bytes, no flags
 001bee                                           68 74  >ht<
 001bf0 74 70 3a 2f 2f 6d 75 73 69 63 62 72 61 69 6e 7a  >tp://musicbrainz<
 001c00 2e 6f 72 67 00 61 30 30 37 35 34 66 30 2d 66 34  >.org.a00754f0-f4<
 001c10 66 34 2d 34 65 37 66 2d 39 63 66 37 2d 37 32 38  >f4-4e7f-9cf7-728<
 001c20 30 30 66 62 39 61 34 37 34                       >00fb9a474<
 001c29                            54 53 4f 50 00 00 00  TSOP, 0x27 = 39 bytes, no flags
 001c30 27 00 00
 001c33          01 ff fe 4c 00 65 00 66 00 74 00 20 00  >...L.e.f.t. .<
 001c40 57 00 69 00 6e 00 67 00 20 00 46 00 61 00 73 00  >W.i.n.g. .F.a.s.<
 001c50 63 00 69 00 73 00 74 00 73 00                    >c.i.s.t.s.<
 001c5a                               54 58 58 58 00 00  TXXX, 0x77 = 119 bytes, no flags
 001c60 00 77 00 00
 001c64             01 ff fe 4d 00 75 00 73 00 69 00 63  >.w.....M.u.s.i.c<
 001c70 00 42 00 72 00 61 00 69 00 6e 00 7a 00 20 00 41  >.B.r.a.i.n.z. .A<
 001c80 00 6c 00 62 00 75 00 6d 00 20 00 49 00 64 00 00  >.l.b.u.m. .I.d..<
 001c90 00 ff fe 66 00 61 00 30 00 63 00 35 00 33 00 35  >...f.a.0.c.5.3.5<
 001ca0 00 63 00 2d 00 38 00 62 00 66 00 61 00 2d 00 34  >.c.-.8.b.f.a.-.4<
 001cb0 00 37 00 39 00 35 00 2d 00 61 00 32 00 36 00 64  >.7.9.5.-.a.2.6.d<
 001cc0 00 2d 00 39 00 39 00 63 00 62 00 36 00 63 00 38  >.-.9.9.c.b.6.c.8<
 001cd0 00 34 00 39 00 37 00 65 00 37 00                 >.4.9.7.e.7.<
 001cdb                                  54 50 55 42 00  TPUB, 0x1b = 27 bytes, no flags
 001ce0 00 00 1b 00 00
 001ce5                01 ff fe 53 00 6b 00 79 00 72 00  >...S.k.y.r.<
 001cf0 61 00 74 00 20 00 4d 00 75 00 73 00 69 00 63 00  >a.t. .M.u.s.i.c.<
 001d00 54 58 58 58 00 00 00 4d 00 00                    TXXX, 0x4d = 77 bytes, no flags
 001d0a                               01 ff fe 4d 00 75  >...M.u<
 001d10 00 73 00 69 00 63 00 42 00 72 00 61 00 69 00 6e  >.s.i.c.B.r.a.i.n<
 001d20 00 7a 00 20 00 41 00 6c 00 62 00 75 00 6d 00 20  >.z. .A.l.b.u.m. <
 001d30 00 52 00 65 00 6c 00 65 00 61 00 73 00 65 00 20  >.R.e.l.e.a.s.e. <
 001d40 00 43 00 6f 00 75 00 6e 00 74 00 72 00 79 00 00  >.C.o.u.n.t.r.y..<
 001d50 00 ff fe 55 00 53 00                             >...U.S.
 001d57                      54 58 58 58 00 00 00 47 00  TXXX, 0x47 = 71 bytes, no flags
 001d60 00
 001d61    01 ff fe 4d 00 75 00 73 00 69 00 63 00 42 00  >....M.u.s.i.c.B.<
 001d70 72 00 61 00 69 00 6e 00 7a 00 20 00 41 00 6c 00  >r.a.i.n.z. .A.l.<
 001d80 62 00 75 00 6d 00 20 00 53 00 74 00 61 00 74 00  >b.u.m. .S.t.a.t.<
 001d90 75 00 73 00 00 00 ff fe 6f 00 66 00 66 00 69 00  >u.s.....o.f.f.i.<
 001da0 63 00 69 00 61 00 6c 00
 001da8                         00 00 00 00 00 00 00 00  >........<
 001db0 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00  >................<
 *
 001f70 00
 001f71    ff fb ...
 \endcode
 *
 *
 *
 \code

|     no. |   offset | tag  |  size | size |       | size+hdr |        |
|---------+----------+------+-------+------+-------+----------+--------|
|       1 | 0x000f71 | TIT2 |  0x31 |   49 | n/a   |       59 |  0xfac |
|       2 | 0x000fac | TPE1 |  0x27 |   39 | n/a   |       49 |  0xfdd |
|       3 | 0x000fdd | TRCK |  0x05 |    5 | n/a   |       15 |  0xfec |
|       4 | 0x000fec | TALB |  0x1b |   27 | n/a   |       37 | 0x1011 |
|       5 | 0x001011 | TPOS |  0x04 |    4 | n/a   |       14 | 0x101f |
|       6 | 0x00101f | TDRC |  0x17 |   23 | n/a   |       33 | 0x1040 |
|       7 | 0x001040 | TCON |  0x0b |   11 | n/a   |       21 | 0x1055 |
|       8 | 0x001055 | TMED |  0x1d |   29 | n/a   |       39 | 0x107c |
|       9 | 0x00107c | TXXX |  0x1b |   27 | n/a   |       37 | 0x10a1 |
|      10 | 0x0010a1 | TXXX |  0x65 |  101 | n/a   |      111 | 0x1110 |
|      11 | 0x001110 | TXXX |  0x3d |   61 | n/a   |       71 | 0x1157 |
|      12 | 0x001157 | TXXX |  0x85 |  133 | $0105 |      143 | 0x11e6 |
|      13 | 0x0011e6 | TXXX |  0x79 |  121 | n/a   |      131 | 0x1269 |
|      14 | 0x001269 | TDOR |  0x17 |   23 | n/a   |       33 | 0x128a |
|      15 | 0x00128a | TSO2 |  0x27 |   39 | n/a   |       49 | 0x12bb |
|      16 | 0x0012bb | TPE2 |  0x27 |   39 | n/a   |       49 | 0x12ec |
|      17 | 0x0012ec | TXXX |  0x87 |  135 | $0107 |      145 | 0x137d |
|      18 | 0x00137d | APIC | 0x85d | 2141 | $105d |     2151 | 0x1be4 |
|      19 | 0x001be4 | UFID |  0x3b |   59 | n/a   |       69 | 0x1c29 |
|      20 | 0x001c29 | TSOP |  0x27 |   39 | n/a   |       49 | 0x1c5a |
|      20 | 0x001c5a | TXXX |  0x77 |  119 | n/a   |      129 | 0x1cdb |
|      21 | 0x001cdb | TPUB |  0x1b |   27 | n/a   |       37 | 0x1d00 |
|      22 | 0x001d00 | TXXX |  0x4d |   77 | n/a   |       87 | 0x1d57 |
|      23 | 0x001d57 | TXXX |  0x47 |   71 | n/a   |       81 | 0x1da8 |
|---------+----------+------+-------+------+-------+----------+--------|
|         |          |      |       | 3399 |       |     3639 |        |
| padding |      457 |      |       |      |       |     4096 |        |
|---------+----------+------+-------+------+-------+----------+--------|
#+TBLFM: @<<$7..@>>>$7=$5+10
#+TBLFM: @>>$5=vsum(@<<$5..@>>>$5)
#+TBLFM: @>>$7=vsum(@<<$7..@>>>$7)
#+TBLFM: @>$7=@>>$7+@>$2
#+TBLFM: @<<$8..@>>>$8='(format "0x%x" (+ (string-to-number $7) (string-to-number (substring $2 2) 16)))

 \endcode
 *
 *
 */

BOOST_AUTO_TEST_CASE( test_multi_id3v2 )
{
  using namespace std;
  using namespace scribbu;

  const fs::path DATA1(get_data_directory() / "lunch4bfast.mp3");
  const fs::path DATA2(get_data_directory() / "lunch4bfast2.mp3");
  const fs::path DATA3(get_data_directory() / "rock-the-joint.id3v2.3.tag");
  const fs::path DATA4(get_data_directory() / "duplicate_id3v2.mp3");

  fs::ifstream ifs1(DATA1, fs::ifstream::binary);

  vector<unique_ptr<id3v2_tag>> tags;
  BOOST_CHECK_THROW( read_all_id3v2(ifs1, back_inserter(tags)), scribbu::error );
  BOOST_CHECK(1 == tags.size());

  fs::ifstream ifs2(DATA2, fs::ifstream::binary);

  tags.erase(tags.begin(), tags.end());
  read_all_id3v2(ifs2, back_inserter(tags));
  BOOST_CHECK(2 == tags.size());

  fs::ifstream ifs3(DATA3, fs::ifstream::binary);

  tags.erase(tags.begin(), tags.end());
  read_all_id3v2(ifs3, back_inserter(tags));
  BOOST_CHECK(1 == tags.size());

  fs::ifstream ifs4(DATA4, fs::ifstream::binary);

  tags.erase(tags.begin(), tags.end());
  read_all_id3v2(ifs4, back_inserter(tags));
  BOOST_CHECK(2 == tags.size());

  stringstream stm0;
  stm0 << *(tags[0]);
  const string GOLDEN0(R"(ID3v2.3(.0) Tag:
3933 bytes, synchronised
flags: 0x00
ArtistXXXX - TitleXXXX
AlbumXXXX, <no year>
TALB: AlbumXXXX
TPE1: ArtistXXXX
TIT2: TitleXXXX
frame PRIV (39 bytes)
frame PRIV (138 bytes)
frame PRIV (20 bytes)
frame PRIV (39 bytes)
frame PRIV (34 bytes)
frame PRIV (31 bytes)
frame PRIV (41 bytes)
3426 bytes of padding
)");

  string text = stm0.str();
  BOOST_CHECK(GOLDEN0 == text);

  stringstream stm1;
  stm1 << *(tags[1]);
  text = stm1.str();
  BOOST_TEST_MESSAGE(text);

  const string GOLDEN1(R"(ID3v2.4(.0) Tag:
4096 bytes, synchronised
flags: 0x00
Left Wing Fascists - Jo Ones Ugly After 2 AM
All Fired Up (track 7/11), <no year>
Content-type Punk
TIT2: Jo Ones Ugly After 2 AM
TPE1: Left Wing Fascists
TRCK: 7/11
TALB: All Fired Up
TPOS: 1/1
frame TDRC (23 bytes)
TCON: Punk
TMED: Digital Media
TXXX: SCRIPT
Latn
TXXX: Acoustid Id
45be336e-51da-4fb8-9d5d-11c52d7d80a1
TXXX: MusicBrainz Album Type
album
TXXX: MusicBrainz Album Artist Id
1fc55b7b-dec1-480f-8599-5c1ac8313c06
TXXX: MusicBrainz Artist Id
1fc55b7b-dec1-480f-8599-5c1ac8313c06
frame TDOR (23 bytes)
frame TSO2 (39 bytes)
TPE2: Left Wing Fascists
TXXX: MusicBrainz Release Group Id
1b4fb8df-a3ca-4d00-b22b-8a5a2282b150
frame APIC (2141 bytes)
UFID: http://musicbrainz.org
      61303037353466302d663466342d346537662d396366372d373238303066623961343734
frame TSOP (39 bytes)
TXXX: MusicBrainz Album Id
fa0c535c-8bfa-4795-a26d-99cb6c8497e7
TPUB: Skyrat Music
TXXX: MusicBrainz Album Release Country
US
TXXX: MusicBrainz Album Status
official
457 bytes of padding
)");
  BOOST_CHECK(GOLDEN1 == text);
}

BOOST_AUTO_TEST_CASE( test_template_text )
{
  const fs::path TEST_FILE(get_data_directory() / "lorca.mp3");

  using scribbu::template_processor;

  template_processor P1("%(title:output=utf-8&compress) - %A( \\(%(album)\\))?%E");
  std::string S = P1(TEST_FILE);
  BOOST_CHECK("Lorca's Novena - Pogues, The (Hell's Ditch [Expanded] (US Version)).mp3" == S);

}

BOOST_AUTO_TEST_CASE( test_total_size )
{
  using namespace std;
  using namespace scribbu;

  const fs::path TEST_DATA_V2_3(get_data_directory() / "id3v2.3.tag");

  fs::ifstream ifs(TEST_DATA_V2_3, fs::ifstream::binary);

  vector<unique_ptr<id3v2_tag>> tags;
  read_all_id3v2(ifs, back_inserter(tags));

  size_t tot, pad;
  tie(tot, pad) = total_id3v2_size(tags.begin(), tags.end(), false);
  BOOST_CHECK( tot == 452951 );
  BOOST_CHECK( pad == 335921 );


} // End test_total_size.
